import selectorParser from 'postcss-selector-parser';
import valueParser from 'postcss-value-parser';

var name = "postcss-scrollbar";

/* eslint-disable consistent-return */

function postcssScrollbar({
  width = '8px',
  edgeAutohide = false
} = {}) {
  const widthMap = {
    none: '0',
    auto: 'initial',
    thin: width
  };
  const colorMap = {
    auto: 'initial',
    dark: 'initial',
    light: 'initial'
  };

  function processWidth(decl, {
    result,
    Rule,
    Declaration
  }) {
    let {
      parent,
      value: keyword
    } = decl;
    let root = parent.parent;

    if (!isValidWidth(keyword)) {
      return decl.warn(result, 'Invalid value for property `scrollbar-width`. ' + 'Must be one of `auto | thin | none`.', {
        word: keyword
      });
    }

    let processor = selectorParser(selectors => {
      selectors.each(selector => {
        selector.append(selectorParser.pseudo({
          value: '::-webkit-scrollbar'
        }));
      });
    });
    let newRule = new Rule({
      selector: processor.processSync(parent.selector)
    });
    newRule.append(new Declaration({
      prop: 'width',
      value: widthMap[keyword]
    }), new Declaration({
      prop: 'height',
      value: widthMap[keyword]
    }));
    root.insertBefore(parent, newRule);
    let value = edgeAutohide ? '-ms-autohiding-scrollbar' : keyword === 'none' ? 'none' : 'auto';
    parent.insertBefore(decl, {
      prop: '-ms-overflow-style',
      value
    });
  }

  function processColor(decl, {
    result,
    Rule,
    Declaration
  }) {
    let {
      nodes
    } = valueParser(decl.value);

    if (isInvalidColor(nodes)) {
      return decl.warn(result, 'Invalid value for property `scrollbar-color`. ' + 'Must be one of `auto | dark | light | <color> <color>`.', {
        word: nodes[0].value
      });
    }

    let values = nodes.filter(value => value.type === 'word').reduce((acc, curr, idx) => {
      if (idx >= 1) {
        return { ...acc,
          track: colorMap[curr.value] || curr.value,
          corner: colorMap[curr.value] || curr.value
        };
      }

      return {
        thumb: colorMap[curr.value] || curr.value,
        track: colorMap[curr.value] || curr.value,
        corner: colorMap[curr.value] || curr.value
      };
    }, {});
    let {
      parent
    } = decl;
    let root = parent.parent;
    Object.keys(values).forEach(pseudo => {
      let processor = selectorParser(selectors => {
        selectors.each(selector => {
          selector.append(selectorParser.pseudo({
            value: `::-webkit-scrollbar-${pseudo}`
          }));
        });
      });
      let newRule = new Rule({
        selector: processor.processSync(parent.selector)
      }).append(new Declaration({
        prop: 'background-color',
        value: values[pseudo]
      }));
      root.insertBefore(parent, newRule);
    });
  }

  return {
    postcssPlugin: name,
    Declaration: {
      'scrollbar-width': processWidth,
      'scrollbar-color': processColor
    }
  };
}

function isValidWidth(keyword) {
  return /auto|thin|none/.test(keyword);
}

function isInvalidColor(nodes) {
  return Array.isArray(nodes) && nodes.length === 1 && !/auto|dark|light/.test(nodes[0].value);
}

postcssScrollbar.postcss = true;

export { postcssScrollbar as default };
